import traceback
from time import sleep
from requests_html import HTMLSession
from urllib3.exceptions import MaxRetryError, ConnectionError
from secreepy.logger import Logger
from secreepy.exceptions import StatusCodeException


def get_request(URL, proxies=None, attempt=3):
    def _return(wait_time=1):
        if attempt:
            sleep(wait_time)
            return get_request(URL, proxies, attempt - 1)
        else:
            Logger().log(f'failed to GET request {URL}', error=True)
            print(traceback.format_exc())
            return None

    session = HTMLSession()
    try:
        response = session.get(URL, stream=True, proxies=proxies)
    except (MaxRetryError, ConnectionError):
        return _return()
    except:
        return _return()
    session.close()
    if response.status_code in [404, 500, 502, 503]:
        return _return()
    if response.status_code != 200:
        raise StatusCodeException(f'Requesting URL {URL} returned {response.status_code} response!')
    return response



from secreepy import Web, utils


class HtmlElement:
    def __init__(self, el):
        if isinstance(el, HtmlElement):
            self.el = el.el
        else:
            self.el = el

    @property
    def text(self):
        ret = self.el.text
        while len(ret) and (ret[0] == '\n' or ret[0] == ' '):
            ret = ret[1:]
        while len(ret) and (ret[-1] == '\n' or ret[-1] == ' '):
            ret = ret[: -1]
        return ret

    def get_attribute(self, attribute):
        return self.el.attrs[attribute]


class Html(Web):
    def __init__(self, proxies=None):
        super().__init__()
        self.html = None
        self.proxies = proxies

    def get(self, url):
        self.html = None
        self.page_source = None
        response = utils.get_request(url, proxies=self.proxies)
        if response:
            self.html = response.html
            self.page_source = response.text
        self.url = url

    def find_element_by_id(self, _id):
        return self.find_element_by_xpath(f'//*[@id="{_id}"]')

    def find_element_by_xpath(self, xpath):
        return HtmlElement(self.html.xpath(xpath)[0])

    def find_elements_by_xpath(self, xpath):
        return [HtmlElement(el) for el in self.html.xpath(xpath)]

    def find_elements_by_id(self, _id):
        return self.find_elements_by_xpath(f'//*[@id="{_id}"]')

    def find_element_by_name(self, name):
        return self.find_element_by_xpath(f'//*[@name="{name}"]')

    def refresh(self):
        self.get(self.url)

    @property
    def title(self):
        return self.find_element_by_xpath('//title').text

    def quit(self):
        pass



import abc


class Web:
    def __init__(self):
        self.url = None
        self.page_source = None

    @abc.abstractmethod
    def get(self, url):
        pass

    @abc.abstractmethod
    def find_element_by_id(self, _id):
        pass

    @abc.abstractmethod
    def find_element_by_xpath(self, xpath):
        pass

    @abc.abstractmethod
    def find_elements_by_xpath(self, xpath):
        pass

    @abc.abstractmethod
    def find_elements_by_id(self, _id):
        pass

    @abc.abstractmethod
    def find_element_by_name(self, name):
        pass

    @abc.abstractmethod
    def refresh(self):
        pass

    def invalid_page(self):
        return self.page_source is None

    @property
    @abc.abstractmethod
    def title(self):
        pass

    @property
    def current_url(self):
        return self.url



import psutil
import os
from datetime import datetime


class Bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


class Logger:
    def __init__(self, debug=0):
        self.debug = debug

    def log(self, *argv, end='\n', error=False, debug=False):
        if debug and not self.debug:
            return
        process = psutil.Process(os.getpid())
        memory = str(process.memory_info().rss // 1024 // 1024) + ' MB'
        now = datetime.now().replace(microsecond=0)
        print(f'{Bcolors.BOLD}{Bcolors.OKBLUE}{memory} {Bcolors.OKGREEN}{now}:{Bcolors.ENDC}', end=' ')
        if error:
            print(f'{Bcolors.BOLD}{Bcolors.FAIL}WARNING:{Bcolors.ENDC}', end=' ')
            for arg in argv:
                print(f'{Bcolors.BOLD}{Bcolors.FAIL}{arg}{Bcolors.ENDC}', end=' ')
        else:
            for arg in argv:
                print(arg, end=' ')
        print('', end=end)



from multiprocessing.pool import ThreadPool
from multiprocessing.context import TimeoutError
from secreepy.exceptions import TooLongException


def timeout(seconds, exception=TooLongException):
    def timeout_decorator(func):
        def _new_func(oldfunc, result, oldfunc_args, oldfunc_kwargs):
            result.append(oldfunc(*oldfunc_args, **oldfunc_kwargs))

        def _(*args, **kwargs):
            result = []

            pool = ThreadPool(1)
            try:
                pool.apply_async(_new_func, args=(func, result, args, kwargs)).get(timeout=seconds)
            except TimeoutError:
                raise exception('function run too long, timeout %d seconds.' % seconds)

            return result[0]

        _.__name__ = func.__name__
        _.__doc__ = func.__doc__
        return _

    return timeout_decorator



from distutils.core import setup
setup(
  name='SeCreepy',
  packages=['secreepy'],
  version='0.4.0',
  license='MIT',
  description='A wrapper for undetected chromedriver that works perfectly for long-running scrapping',
  author_email='fadi.younes.syr@gmail.com',
  url='https://github.com/repeating/secreepy',
  download_url='https://github.com/repeating/secreepy/archive/v0.4.0.tar.gz',
  keywords=['Selenium', 'undetected chromedriver', 'Web Scrapping'],
  install_requires=[
          'selenium',
          'undetected-chromedriver',
          'urllib3',
          'psutil'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
  ],
)
